/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef BIT_FIELD_H
#define BIT_FIELD_H

#include <string>
#include <vector>

#include <libxml++/libxml++.h>

#include <bit/fieldbase.h>

namespace bit
  {

  /**
   * A Field identifies a starting point and a length.
   *
   * This class extends FieldBase by including a name and a description.
   *
   * @ingroup records
   * @author Rick L. Vinyard, Jr.
   */
  class Field: public FieldBase
    {
    public:

      typedef BitPointer<Field> pointer;

      Field( std::string name="",
             size_t length=0,
             size_t length_units=BITS,
             int offset=0,
             size_t offset_units=BITS,
             std::string descr=""
           );

      static Field::pointer create( std::string name="",
                                    size_t length=0,
                                    size_t length_units=BITS,
                                    int offset=0,
                                    size_t offset_units=BITS,
                                    std::string descr=""
                                  );

      virtual ~Field();

//       size_t read(xmlpp::Element& xml_element, size_t last_position);

      /** @return the length of this object in it's length units */
      virtual size_t length() const;

      virtual size_t length( size_t units ) const;

      virtual void set_length(size_t length);

      virtual size_t length_units() const;

      virtual void set_length_units(size_t units);

      /** @return the offset of this object in the offset units of this object */
      virtual int offset() const;

      virtual int offset( size_t units ) const;

      virtual void set_offset(int offset);

      virtual size_t offset_units() const;

      virtual void set_offset_units(size_t units);

      /**
       * Returns the string name associated with this object
       *
       * This pure virtual method is reimplemented by children to
       * provide the proper notation, such as the array notation of
       * elements, by children.
       */
      virtual std::string name() const;

      virtual std::string name(int depth) const;

      virtual void set_name(const std::string& name);

      /**
       * Returns the string description of this object.
       */
      virtual std::string description() const;

      virtual void set_description(const std::string& description);

      virtual const std::string& xml();

      virtual FieldBase::pointer clone();

      virtual Container container_type() { return FIELD; }

    protected:
      std::string m_name;
      size_t m_length;
      size_t m_length_units;
      int m_offset;
      size_t m_offset_units;
      std::string m_description;
    };

}

#endif
