/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "record.h"

#include <sstream>
#include <algorithm>

#include <ctype.h>

#define WHOLE_UNITS(__offset, __units) \
  ( ((__offset) > 0.0) ? ( (__offset)/__units+(((__offset)%__units)?1:0) ) : ( (__offset)/__units+(((__offset)%__units)?(-1):0)) )

namespace bit
  {

  Record::Record(std::string name, size_t length_units, int offset, size_t offset_units, std::string description):
      RecordBase(name, length_units, offset, offset_units, description),
  m_fields(*this)
  {
    set_name(name);
  }

  Record::pointer Record::create(std::string name, size_t length_units, int offset, size_t offset_units, std::string description)
  {
    return Record::pointer( new Record(name, length_units, offset, offset_units, description) );
  }

  Record::~Record()
  {}

  size_t Record::length( ) const
  {
    size_t length = 0;
    RecordStorage::Fields::const_iterator i;

    for (i = m_fields.field_set.begin(); i != m_fields.field_set.end(); i++)
    {
      // get the length of each child as its internal offset + its length in the length_units of this record
      size_t bits = (*i)->offset(BITS) + (*i)->length(BITS);
      length = std::max(length, bits);
    }

    return WHOLE_UNITS(length, m_length_units);
  }

  size_t Record::length( size_t units ) const
  {
    return this->FieldBase::length(units);
  }

  const std::string & Record::xml( )
  {
    std::ostringstream sout;
    sout << "<record name=\"" << m_name
        << "\" length_units=\"" << m_length_units
        << "\" start=\"" << m_offset
        << "\" start_units=\"" << m_offset_units << "\">\n";

    if ( m_type.type() != TYPE_NONE )
      sout << m_type.xml() << "\n";

    iterator i;
    for ( i = m_fields.begin(); i != m_fields.end(); i++ )
      sout << i->xml() << "\n";

    sout << "</record>\n";
    m_xml = sout.str();
    return m_xml;
  }

  FieldBase::iterator Record::begin()
  {
    return m_fields.begin();
  }

  FieldBase::iterator Record::end()
  {
    return m_fields.end();
  }

  FieldBase::pointer Record::field(std::string s) {
    return m_fields.field(s);
  }

  FieldBase::pointer Record::field(size_t i) {
    return m_fields.field(i);
  }

  void Record::add_field(FieldBase::pointer field)
  {
    m_fields.add_field(field);
  }

  void Record::remove_field(const std::string& name)
  {
    m_fields.remove_field(name);
  }

  void Record::remove_field(FieldBase::pointer field)
  {
    m_fields.remove_field(field);
  }

  FieldBase::pointer Record::previous_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {
    return m_fields.previous_field(current_field);
  }

  FieldBase::pointer Record::next_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {
    return m_fields.next_field(current_field);
  }

  size_t Record::fields() {
    return m_fields.fields();
  }

  FieldBase::pointer Record::clone() {
    Record::pointer record = Record::create();
    *record = *this;
    record->m_parent = NULL;
    return record;
  }


}


//   std::string Record::get_xml( )
//   {
//     std::ostringstream sout;
//     sout << "<record name=\"" << m_name << "\""
//     << " start=\"" << m_offset << "\"";
//
//     if (m_size > 1)
//       sout << " vector_size=\"" << m_size << "\"";
//
//     if (m_description.size() > 0)
//       sout << "><description>" << m_description << "</description>";
//     else
//       sout << ">";
//     if (m_is_vector)
//       {
//         Record& fg = dynamic_cast<Record&>(*(m_fields[0]));
//         sout << fg.children_xml();
//       }
//     else
//       sout << children_xml();
//     sout << "</record>";
//     return sout.str();
//   }

//   std::string Record::children_xml( )
// {
//   std::ostringstream sout;
//   for (int i = 0; i < m_fields.size(); i++)
//     sout << m_fields[i]->get_xml();
//   return sout.str();
// }

